/*
 * Copyright (c) 2005-2010 KOM � Multimedia Communications Lab
 *
 * This file is part of Simonstrator.KOM.
 * 
 * Simonstrator.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tudarmstadt.maki.simonstrator.api.uavsupport;

import java.util.UUID;

import de.tudarmstadt.maki.simonstrator.api.component.network.NetworkComponent.NetInterfaceName;
import de.tudarmstadt.maki.simonstrator.api.component.overlay.OverlayContact;
import de.tudarmstadt.maki.simonstrator.api.component.overlay.OverlayMessage;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.location.Location;
import de.tudarmstadt.maki.simonstrator.api.uavsupport.communication.CommunicationInterfaceListener;
import de.tudarmstadt.maki.simonstrator.api.uavsupport.communication.ConnectionInformation;
import de.tudarmstadt.maki.simonstrator.api.uavsupport.communication.UAVToBaseInterface;


/**
 * Interface for control instances on the UAS base station.
 * 
 * @author Julian Zobel
 *
 */
public interface BaseControl extends CommunicationInterfaceListener {

	/**
	 * Send a message through the respective network.
	 * 
	 * @param msg
	 *            The message
	 * @param network
	 *            The network (name of the network interface) to use
	 * @param broadcast
	 *            Broadcast this message?
	 */
	public void sendMessage(OverlayMessage msg, NetInterfaceName network, boolean broadcast);

	/**
	 * Retrieve an unspecified UAV ({@link UAVToBaseInterface}) from the
	 * available set of *ready* and unassigned UAVs at that base station.
	 * 
	 * @return
	 */
	public UAVToBaseInterface retrieveUAV();

	/**
	 * Retrieve an unspecified UAV ({@link UAVToBaseInterface}) from the
	 * available set of *ready* UAVs at that base station. Also includes ready
	 * UAVs that are assigned (previously *blocked*) by the application.
	 * 
	 * @param applicationID
	 *            The identifier of the application.
	 * @return
	 */
	public UAVToBaseInterface retrieveUAV(UUID applicationID);

	/**
	 * Retrieve a specific UAV ({@link UAVToBaseInterface}) from the available
	 * set of *ready* UAVs or UAVs that are ready and assigned to the given
	 * application.
	 * 
	 * @param cInfo
	 *            Connection Information of the UAV.
	 * @param applicationID
	 *            The identifier of the application.
	 * @return
	 */
	public UAVToBaseInterface retrieveUAV(ConnectionInformation cInfo, UUID applicationID);

	/**
	 * Block an unspecified UAV ({@link UAVToBaseInterface}) from the available
	 * set of *ready* and unassigned UAVs at that base station and assign it to
	 * the given application. The UAV cannot be retrieved by other applications.
	 * 
	 * @param applicationID
	 *            The identifier of the application.
	 * @return
	 */
	public UAVToBaseInterface blockUAV(UUID applicationID);

	/**
	 * Block a specific UAV ({@link UAVToBaseInterface}) that is currently ready
	 * and unassigned at that base station and assign it to the given
	 * application. The UAV cannot be retrieved by other applications.
	 * 
	 * @param cInfo
	 *            Connection Information of the UAV.
	 * @param applicationID
	 *            The identifier of the application.
	 * @return
	 */
	public UAVToBaseInterface blockUAV(ConnectionInformation cInfo, UUID applicationID);

	/**
	 * Free/unblock/unassign a specific UAV that is currently assigned to the
	 * given application. Does not require that the UAV is currently connected.
	 * 
	 * @param cInfo
	 *            Connection Information of the UAV.
	 * @param applicationID
	 *            The identifier of the application.
	 * @return True if the UAV was successfully freed, false otherwise.
	 */
	public boolean freeUAV(ConnectionInformation cInfo, UUID applicationID);

	/**
	 * Check a specific UAV to be connected to that base station and ready for
	 * assignment.
	 * 
	 * @param cInfo
	 *            Connection Information of the UAV.
	 * @return True if ready and available, false otherwise.
	 */
	public boolean isReadyUAV(ConnectionInformation cInfo);

	public OverlayContact getOverlayContact(NetInterfaceName network);

	public Location getBaseLocation();

	/**
	 * @return Number of UAVs, that are assigned to this base station (includes
	 *         available, blocked, and operating UAVs).
	 */
	public int getNumberOfAssignedUAVs();

	/**
	 * @return Number of UAVs, that are currently connected to that base station
	 *         and ready for assignment.
	 */
	public int getNumberOfReadyUAVs();

}
